# encoding: UTF-8

require 'spec_helper.rb'
require 'bigdecimal'
require 'rational'

describe 'when serializing' do
  before :each do
    AMF::Root.clear_class_aliases
  end


  describe 'AMF3' do
    describe 'simple messages' do
      it 'should serialize a null' do
        compare_with_fixture(nil, 'simple/amf3-null.bin')
      end

      it 'should serialize a false' do
        compare_with_fixture(false, 'simple/amf3-false.bin')
      end

      it 'should serialize a true' do
        compare_with_fixture(true, 'simple/amf3-true.bin')
      end

      it 'should serialize integers' do
        compare_with_fixture(AMF::AMFConstants::INTEGER_MAX, 'simple/amf3-max.bin')
        compare_with_fixture(0, 'simple/amf3-0.bin')
        compare_with_fixture(AMF::AMFConstants::INTEGER_MIN, 'simple/amf3-min.bin')
      end

      it 'should serialize large integers' do
        compare_with_fixture(AMF::AMFConstants::INTEGER_MAX + 1, 'simple/amf3-large-max.bin')
        compare_with_fixture(AMF::AMFConstants::INTEGER_MIN - 1, 'simple/amf3-large-min.bin')
      end

      it 'should serialize floats' do
        compare_with_fixture(3.5, 'simple/amf3-float.bin')
      end

      it 'should serialize BigNums' do
        compare_with_fixture(2**1000, 'simple/amf3-bigNum.bin')
      end

      it 'should serialize float Numeric conformers' do
        compare_with_fixture(Rational(7, 2), 'simple/amf3-float.bin')
      end

      it 'should serialize a simple string' do
        compare_with_fixture('String . String', 'simple/amf3-string.bin')
      end

      it 'should serialize a frozen string' do
        compare_with_fixture('String . String'.freeze, 'simple/amf3-string.bin')
      end

      it 'should serialize a symbol as a string' do
        compare_with_fixture(:foo, 'simple/amf3-symbol.bin')
      end
    end

    describe 'objects' do

      it 'should serialize Date objects' do
        compare_with_fixture(Date.civil(1970, 1, 1, 0), 'complex/amf3-date.bin')
      end

      it 'should serialize Time objects' do
        compare_with_fixture(Time.utc(1970, 1, 1, 0), 'complex/amf3-date.bin')
      end

      it 'should serialize DateTime objects' do
        compare_with_fixture(DateTime.civil(1970, 1, 1, 0), 'complex/amf3-date.bin')
      end

      it 'should serialize an unmapped object as a dynamic anonymous object' do
        class NonMappedObject
          def another_public_property
            'a_public_value'
          end

          attr_accessor :nil_property
          attr_accessor :property_one
          attr_writer :read_only_prop

          def method_with_arg(arg = 'foo')
            arg
          end
        end
        obj              = NonMappedObject.new
        obj.property_one = 'foo'
        obj.nil_property = nil

        compare_with_fixture(obj, 'complex/amf3-dynamic-object.bin')
      end

      it 'should serialize a hash as a dynamic anonymous object' do
        hash          = {}
        hash[:answer] = 42
        hash['foo']   = 'bar'

        compare_with_fixture(hash, 'complex/amf3-hash.bin')
      end

      it 'should serialize an empty array' do
        compare_with_fixture([], 'complex/amf3-empty-array.bin')
      end

      it 'should serialize an array of primitives' do
        compare_with_fixture([1, 2, 3, 4, 5], 'complex/amf3-primitive-array.bin')
      end

      it 'should serialize an array of mixed objects' do
        value0 = {:foo_one => 'bar_one'}
        value1 = {:foo_two => ''}
        class SimpleObj
          attr_accessor :foo_three
        end

        value2           = SimpleObj.new
        value2.foo_three = 42

        input = [value0, value1, value2, {}, [value0, value1, value2], [], 42, '', [], '', {}, 'bar_one', value2]

        compare_with_fixture(input, 'complex/amf3-mixed-array.bin')
      end


      it 'should serialize a byte array' do
        str = "\000\003これtest\100"
        str.force_encoding('ASCII-8BIT') if str.respond_to?(:force_encoding)
        input = StringIO.new(str)

        compare_with_fixture(input, 'complex/amf3-byte-array.bin')
      end
    end

    describe 'and implementing the AMF Spec' do
      it 'should keep references of duplicate strings' do
        class StringCarrier
          attr_accessor :str
        end
        foo = 'foo'
        bar = 'str'

        sc     = StringCarrier.new
        sc.str = foo

        compare_with_fixture([foo, bar, foo, bar, foo, sc], 'references/amf3-string-ref.bin')
      end

      it 'should not reference the empty string' do
        compare_with_fixture(['', ''], 'references/amf3-empty-string-ref.bin')
      end

      it 'should keep references of duplicate dates' do
        input = Time.utc 1970, 1, 1, 0

        compare_with_fixture([input, input], 'references/amf3-date-ref.bin')
      end

      it 'should keep reference of duplicate objects' do
        class SimpleReferenceableObj
          attr_accessor :foo
        end
        value0     = SimpleReferenceableObj.new
        value0.foo = :bar

        value1     = SimpleReferenceableObj.new
        value1.foo = value0.foo

        compare_with_fixture([[value0, value1], 'bar', [value0, value1]], 'references/amf3-object-ref.bin')

      end

      it 'should keep reference of duplicate object traits' do
        value0     = RubyClass.new
        value0.foo = 'foo'

        def value0.encode_amf(serializer)
          serializer.write_object(self, nil, {:class_name => 'org.amf.ASClass', :dynamic => false, :externalizable => false, :members => %w( baz foo )})
        end

        value1     = RubyClass.new
        value1.foo = 'bar'

        def value1.encode_amf(serializer)
          serializer.write_object(self, nil, {:class_name => 'org.amf.ASClass', :dynamic => false, :externalizable => false, :members => %w( baz foo )})
        end

        compare_with_fixture([value0, value1], 'references/amf3-trait-ref.bin')
      end

      it 'should keep references of duplicate arrays' do
        a = [1, 2, 3]
        b = %w{ a b c }

        compare_with_fixture([a, b, a, b], 'references/amf3-array-ref.bin')
      end

      it 'should not keep references of duplicate empty arrays unless the object_id matches' do
        a = []
        b = []
        expect(a).to eq(b)
        expect(a.object_id).not_to eq(b.object_id)

        compare_with_fixture([a, b, a, b], 'references/amf3-empty-array-ref.bin')
      end

      it 'should keep references of duplicate byte arrays' do
        value = StringIO.new 'ASDF'

        compare_with_fixture([value, value], 'references/amf3-byte-array-ref.bin')
      end

      it 'should serialize a deep object graph with circular references' do
        class GraphMember
          attr_accessor :children
          attr_accessor :parent

          def initialize
            self.children = []
          end

          def add_child(child)
            children << child
            child.parent = self
            child
          end
        end

        input = GraphMember.new
        #level_1_child_1
        input.add_child(GraphMember.new)
        #level_1_child_2
        input.add_child(GraphMember.new)

        compare_with_fixture(input, 'references/amf3-graph-member.bin')
      end
    end

    describe 'and handling encodings', if: ''.respond_to?(:force_encoding) do
      it 'should support multiple encodings' do
        shift_str = "\x53\x68\x69\x66\x74\x20\x83\x65\x83\x58\x83\x67".force_encoding('Shift_JIS') # 'Shift テスト'
        utf_str   = "\x55\x54\x46\x20\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88".force_encoding('UTF-8') # 'UTF テスト'

        compare_with_fixture([5, shift_str, utf_str, 5], 'encoding/amf3-complex-encoded-string-array.bin')
      end

      it 'should keep references of duplicate strings with different encodings' do
        # String is 'this is a テスト'
        shift_str = "\x74\x68\x69\x73\x20\x69\x73\x20\x61\x20\x83\x65\x83\x58\x83\x67".force_encoding('Shift_JIS')
        utf_str   = "\x74\x68\x69\x73\x20\x69\x73\x20\x61\x20\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88".force_encoding('UTF-8')

        compare_with_fixture([shift_str, utf_str], 'encoding/amf3-encoded-string-ref.bin')
      end

      it 'should handle inappropriate UTF-8 characters in byte arrays' do
        str = "\xff\xff\xff".force_encoding('ASCII-8BIT')
        str.freeze # For added amusement
        output = AMF::Root.serialize(StringIO.new(str))
        expect(output).to eq("\x0c\x07\xff\xff\xff".force_encoding('ASCII-8BIT'))
      end
    end
  end
end